#!/bin/bash 

# this script changes the branches of the assets submodule located in projects/lapis_assets based on the build environment that is passed as a parameter
# the commit number to be used by each build environment is kept in projects/Lapis/Resources/asset_commits.txt

# parameters, this script takes:
# 1) build type : $ARG_DEV, $ARG_STAG_NR, $ARG_STAG_CURR, $ARG_PRE_PROD, $ARG_PROD
#		$ARG_DEV - when building from target LapisDEV_SG and uses the commit# described by $KEY_DEV in asset_commits.txt
#		$ARG_STAG_NR - when building from target LapisStaging_SG and uses the commit# described by $KEY_STAG_NR in asset_commits.txt
#		$ARG_STAG_CURR - when building from target LapisStagingCurr_SG and uses the commit# described by $KEY_STAG_CURR in asset_commits.txt
#		$ARG_PRE_PROD - when building from target LapisProd_SG or LapisProdSQEX_SG and uses the commit# described by $KEY_PRE_PROD in asset_commits.txt
#		$ARG_PROD - (currently does not have a build target set) Uses the commit# described by $KEY_PRE_PROD in asset_commits.txt
# 2) whether the asset repository should be cleaned.
#		pass in '$ARG_CLEAN' if the git submodule needs to be reset and $ARG_NO_CLEAN if no

#Constants
ARG_DEV='DEV'
ARG_STAG_NR='STAG_NR'
ARG_STAG_CURR='STAG_CURR'
ARG_PRE_PROD='PRE_PROD'
ARG_PROD='PROD'

ARG_CLEAN='clean'
ARG_NO_CLEAN='no-clean'

KEY_DEV='lapis_dev'
KEY_STAG_NR='lapis_staging'
KEY_STAG_CURR='lapis_staging_curr'
KEY_PRE_PROD='lapis_pre_production'
KEY_PROD='lapis_production'

PATH_RELATIVE_LAPIS_ASSETS="../../"
FOLDER_NAME_LAPIS_ASSETS="lapis_assets/"
FILENAME_ASSET_COMMITS="asset_commits.txt"

#no of arguments check
if [ "$#" -ne 2 ]; then
    echo "You must enter exactly 2 command line arguments"
    echo "e.g. update_asset_branch.sh $ARG_DEV $ARG_CLEAN"
    echo "e.g. update_asset_branch.sh $ARG_STAG_NR $ARG_NO_CLEAN"
    exit 1
fi

#transforming the arguments into an array
args=("$@")

#Variables
#storing the commits that are obtained from the assets_commits.txt file
DEV_COMMIT=""
STAGING_COMMIT=""
STAGING_CURR_COMMIT=""
PRE_PROD_COMMIT=""
PROD_COMMIT=""

HOME_DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"$"/"

# loads the assets_commits.txt file
while IFS= read -r var
do
  echo "configuration read - $var"
	IN=$var
	IFS=':' read -ra mails2 <<< "$IN"

	first=true #bool to keep track of whether iteration below is looking at the environment or commit#
	for x in $mails2
	do
		#echo "> [$x]"
		
		if  [ "${first}" = true ]; then
			#checks the key for the commit number
			if [ "${x}" = "$KEY_DEV" ]; then
				DEV_COMMIT=${mails2[1]}
				echo DEV_COMMIT=${mails2[1]}
			elif [ "${x}" = "$KEY_STAG_NR" ]; then
				STAGING_COMMIT=${mails2[1]}
				echo STAGING_COMMIT=${mails2[1]}
			elif [ "${x}" = "$KEY_STAG_CURR" ]; then
				STAGING_CURR_COMMIT=${mails2[1]}
				echo STAGING_CURR_COMMIT=${mails2[1]}
			elif [ "${x}" = "$KEY_PRE_PROD" ]; then
				PRE_PROD_COMMIT=${mails2[1]}
				echo PRE_PROD_COMMIT=${mails2[1]}
			elif [ "${x}" = "$KEY_PROD" ]; then
				PROD_COMMIT=${mails2[1]}
				echo PROD_COMMIT=${mails2[1]}
			else
				echo "The key '${x}' does not match any of '$KEY_DEV' '$KEY_STAG_NR' '$KEY_STAG_CURR' '$KEY_PRE_PROD' or '$KEY_PROD'"
				exit 6
			fi
		fi
		first=false
	done
done < "$HOME_DIR$FILENAME_ASSET_COMMITS"

#checks for the existence of the $FOLDER_NAME_LAPIS_ASSETS
if [ ! -d $PATH_RELATIVE_LAPIS_ASSETS$FOLDER_NAME_LAPIS_ASSETS ]; then
  #if submodule directory doesn't exist, then we create and update it
  echo git submodule directory does not exist. Running \'git submodule init\'
  git submodule init
  echo git submodule init completed
  echo running git submodule update
  git submodule update --depth 1
  echo git submodule update completed
else
	# if submodule directory exists, we check if it is empty and run update if it is.
	if find $PATH_RELATIVE_LAPIS_ASSETS$FOLDER_NAME_LAPIS_ASSETS -mindepth 1 -print -quit | grep -q .; then
		echo submodule folder is not empty, not running \'git submodule update\'
	else
		echo submodule folder is empty, running \'git submodule update\'
		git submodule update --depth 1
		echo git submodule update completed
	fi
fi

#gets the commit number for the supplied parameter
pushd $PATH_RELATIVE_LAPIS_ASSETS$FOLDER_NAME_LAPIS_ASSETS

if [ "${args[0]}" = "$ARG_DEV" ]; then
	commit=$DEV_COMMIT
elif [ "${args[0]}" = "$ARG_STAG_NR" ]; then
	commit=$STAGING_COMMIT
elif [ "${args[0]}" = "$ARG_STAG_CURR" ]; then
	commit=$STAGING_CURR_COMMIT
elif [ "${args[0]}" = "$ARG_PRE_PROD" ]; then
	commit=$PRE_PROD_COMMIT
elif [ "${args[0]}" = "$ARG_PROD" ]; then
	commit=$PROD_COMMIT
else
	echo "The arg ${args[0]} does not match any of '$ARG_DEV' '$ARG_STAG_NR' '$ARG_STAG_CURR' '$ARG_PRE_PROD' or '$ARG_PROD'"
	exit 7
fi

# discards local changes and resets submodule to a particular commit if the parameter $ARG_CLEAN is supplied
# echo $commit
if [ "${args[1]}" = "$ARG_CLEAN" ]; then
	echo "Cleaning git submodule and resetting to commit#$commit"
	echo "asset submodule git clean -f -d"
	git clean -f -d
	echo "asset submodule git clean completed"
	echo "asset submodule git fetch"
	git fetch
	echo "asset submodule git fetch completed"
	echo "asset submodule git reset --hard $commit"
	git reset --hard $commit
	echo "asset submodule git reset completed"
	echo git checkout $commit
	git checkout $commit #just to get a detached head
	echo "asset submodule git checkout completed"
	echo "git process completed"
elif [ "${args[1]}" = "$ARG_NO_CLEAN" ]; then
	echo "not cleaning and not resetting git submodule"
else
	echo "The arg ${args[1]} does not match any of '$ARG_CLEAN' or '$ARG_NO_CLEAN'"
	exit 8
fi

popd